/*:
# **Kapitel 2.4**
## **Einstiegsaufgabe Steinzeit**
Emma hat das Spiel Steinzeit entworfen, bei dem zu Beginn acht Steinfresser erstellt und zufällig auf dem Spielfeld verteilt werden. Nach Spielstart bewegen sie sich auf Kreisbahnen und man muss seinen Stein mit den Pfeiltasten zum rechten Rand steuern, ohne vorher gefressen zu werden. Der Quellcode ist allerdings noch fehlerhaft.\
a \
Behebe die Fehler in der Klasse STEINZEIT, so dass das Spiel funktioniert.\
b \
Furkan äußert vorsichtig Zweifel an Emmas Programmierstil. Beziehe zu Furkans beiden Aussagen begründet Stellung.
*/

/**
Die Spielfigur, ein Stein

- author: Johannes Neumeyer
- version: 1.0
*/
class Stein: Figur
{
    /**
    Legt die Form des Steins fest
    */
    override init()
    {
        super.init()
        FigurteilFestlegenEllipse(x: -50, y: -50, breite: 100, höhe: 100, farbe: "grau")
        PositionSetzen(x: 25, y: 250)
    }

	/**
    Die  Aktionsmethode für gedrückte Sondertasten.
    - parameters:
        - taste: KeyCode der gedrückten Taste
    */
    override func SonderTasteGedrückt (taste: Int)
    {
        switch taste
        {
            // Pfeil nach links
            case 123:
                WinkelSetzen(winkel: 180)
                Gehen(länge: 10)
            // Pfeil nach oben
            case 126:
                WinkelSetzen(winkel: 90)
                Gehen(länge: 10)
            // Pfeil nach rechts
            case 124:
                WinkelSetzen(winkel: 0)
                Gehen(länge: 10)
            // Pfeil nach unten
            case 125:
                WinkelSetzen(winkel: 270)
                Gehen(länge: 10)
            default:
                break
        }
    }
}

/**
Steinfresser

- author: Johannes Neumeyer
- version: 1.0
*/
class Steinfresser: Figur
{
    /**
    Legt die Form des Steinfressers fest
    */
    override init()
    {
        super.init()
        SpielfigurFestlegen()
    }
    
    /**
	Stellt den Steinfresser auf seine Startposition
    */
    func StartpositionEinnehmen()
    {
        let zufallsgenerator = Zufall()
        PositionSetzen(x: zufallsgenerator.ZufallGanz(top: 650) + 100, y: zufallsgenerator.ZufallGanz(top: 450) + 50)
    }
    
    /**
	Bewegt den Steinfresser
    */
    func Bewegen()
    {
        Gehen(länge: 25)
        Drehen(grad: 18)
    }

	/**
	Beschreibt die Form des Steinfressers
    */
    private func SpielfigurFestlegen()
    {
        FigurteilFestlegenEllipse(x: -50, y: -50, breite: 100, höhe: 100, farbe: "blau")
        FigurteilFestlegenEllipse(x: -20, y: 10, breite: 40, höhe: 20, farbe: "rot")
        FigurteilFestlegenEllipse(x: -20, y: -20, breite: 5, höhe: 5, farbe: "gelb")
        FigurteilFestlegenEllipse(x: 20, y: -20, breite: 5, höhe: 5, farbe: "gelb")
    }
}
/**
Spielfeld des Steinfresserspiels

- author: Johannes Neumeyer
- version: 1.0
*/
class Steinzeit: Ereignisbehandlung
{
    private var spielfigur: Stein

    private var sf1: Steinfresser
    private var sf2: Steinfresser
    private var sf3: Steinfresser
    private var sf4: Steinfresser
    private var sf5: Steinfresser
    private var sf6: Steinfresser
    private var sf6: Steinfresser
    private var sf8: Steinfresser

    /**
    Legt das Spielfeld und die Objekte an
    */
    override init()
    {
        spielfigur = Stein()
        // Gegner erstellen
        sf1 = Steinfresser()
        sf2 = Steinfresser()
        sf3 = Steinfresser()
        sf4 = Steinfresser()
        sf6 = Steinfresser()
        sf7 = Steinfresser()
        sf8 = Steinfresser()
        super.init()

        // Alle Gegner sollen ihre (zufällige) Startposition einnehmen
        sf1.StartpositionEinnehmen()
        sf3.StartpositionEinnehmen()
        sf4.StartpositionEinnehmen()
        sf5.StartpositionEinnehmen()
        sf6.StartpositionEinnehmen()
        sf7.StartpositionEinnehmen()
        sf8.StartpositionEinnehmen()

        TaktdauerSetzen(dauer: 100)
        Starten()
    }

    /**
    Überschriebene Methode von Ereignisbehandlung,
    die automatisch bei jedem Taktschlag aufgerufen wird
    */
    override func TaktImpulsAusführen ()
    {
        // Bei jedem Taktschlag sollen sich alle Gegner weiterbewegen
        sf1.Bewegen()
        sf3.Bewegen()
        sf4.Bewegen()
        sf5.Bewegen()
        sf6.Bewegen()

        // Nach jedem Taktschlag muss geprüft worden, ob man gewonnen oder verloren hat
        GewonnenVerlorenPrüfen()
    }

    /**
    Prüft, ob das Spiel gewonnen oder verloren wurde
    und erstellt einen entsprechenden Text
    */
    private func GewonnenVerlorenPrüfen()
    {
        // Gewonnen prüfen
        if spielfigur.XPositionGeben() > 800
        {
            Anhalten()
            let gewonnen = Text()
            gewonnen.TextSetzen(text: "Gewonnen!")
            gewonnen.PositionSetzen(x: 300, y: 250)
            gewonnen.FarbeSetzen(farbe: "grün")
            gewonnen.TextGrößeSetzen(größe: 40)
        }
        // Verloren prüfen
        if (spielfigur.Berührt())
        {
            Anhalten();
            let verloren = Text()
            verloren.TextSetzen(text: "Verloren!")
            verloren.PositionSetzen(x: 300, y: 250)
            verloren.FarbeSetzen(farbe: "rot")
            verloren.TextGrößeSetzen(größe: 40)
        }
    }
}


